@@ -129,6 +129,42 @@ addHandlerRightclick.default <- function(obj, handler, action=NULL, ...)
   obj$add_handler_right_clicked(handler, action=action, ...)
 
 
+##' Add handler for shift click event
+##'
+##' This may not be supported by all toolkits.
+##' @export
+##' @rdname gWidgets-handlers
+addHandlerShiftclick <- function(obj, handler, action=NULL, ...) UseMethod("addHandlerShiftclick")
+
+##' Default S3 method
+##'
+##' @inheritParams addHandler
+##' @export
+##' @rdname gWidgets-handlers
+##' @method addHandlerShiftclick default
+##' @S3method addHandlerShiftclick default
+addHandlerShiftclick.default <- function(obj, handler, action=NULL, ...)
+  obj$add_handler_shift_clicked(handler, action=action, ...)
+
+
+##' Add handler for control+click event
+##'
+##' This may not be supported by all toolkits.
+##' @export
+##' @rdname gWidgets-handlers
+addHandlerControlclick <- function(obj, handler, action=NULL, ...) UseMethod("addHandlerControlclick")
+
+##' Default S3 method
+##'
+##' @inheritParams addHandler
+##' @export
+##' @rdname gWidgets-handlers
+##' @method addHandlerControlclick default
+##' @S3method addHandlerControlclick default
+addHandlerControlclick.default <- function(obj, handler, action=NULL, ...)
+  obj$add_handler_control_clicked(handler, action=action, ...)
+
+
 
 
 ##' Add handler for column click event
@@ -373,7 +409,7 @@ addHandlerIdle <- function( ...) {
 
 ##' Add a "popup" menu to the widget
 ##'
-##' Defaults to adding a 3rd mouse popup menu, better known as a
+##' Defaults to adding a right-click mouse popup menu, better known as a
 ##' context menu, though some toolkits have both this and the latter
 ##' provided. 
 ##' @param menulist a list of \code{gaction} items or a \code{gmenu} instance
@@ -391,7 +427,7 @@ addPopupMenu.default <-  function(obj, menulist, action=NULL, ...)
   obj$add_popup_menu(menulist, action=action, ...)
 
 
-##' Add a 3rd-mouse "popup" menu to the widget
+##' Add a context  "popup" menu to the widget
 ##'
 ##' These menus are also known as context menus, though there isn't
 ##' really a good mechanism within \pkg{gWidgets2} to make the menu
@@ -399,18 +435,31 @@ addPopupMenu.default <-  function(obj, menulist, action=NULL, ...)
 ##' @inheritParams addPopupMenu
 ##' @export
 ##' @rdname gWidgets-handlers
-add3rdmousePopupMenu <- function(obj, menulist, action=NULL, ...) UseMethod("add3rdmousePopupMenu")
+addRightclickPopupMenu <- function(obj, menulist, action=NULL, ...) UseMethod("addRightclickPopupMenu")
 
 ##' S3 method for popup menu
 ##'
 ##' @export
 ##' @rdname gWidgets-handlers
-##' @method add3rdmousePopupMenu default
-##' @S3method add3rdmousePopupMenu default
-add3rdmousePopupMenu.default <-  function(obj, menulist, action=NULL, ...)
+##' @method addRightclickPopupMenu default
+##' @S3method addRightclickPopupMenu default
+addRightclickPopupMenu.default <-  function(obj, menulist, action=NULL, ...)
   obj$add_3rd_mouse_popup_menu(menulist, action=action, ...)
 
 
+##' Alias for poor initial name choice
+##'
+##' @rdname gWidgets-handlers
+##' @export
+##' @aliases addRightclickPopupMenu
+add3rdmousePopupMenu <- addRightclickPopupMenu
+
+##' Alias for poor punctation choice
+##'
+##' @rdname gWidgets-handlers
+##' @export
+##' @aliases addRightclickPopupMenu
+add3rdMousePopupMenu <- add3rdmousePopupMenu
 
 ##' Specify a widget is a source for a drop action
 ##'
